USE [SSIS_DF_TEST]
GO

/****** Object:  StoredProcedure [dbo].[DFswitch]    Script Date: 16/07/2019 20:15:53 ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO


CREATE PROCEDURE [dbo].[DFswitch]
AS
BEGIN

WAITFOR DELAY '00:00:05';   -- delay for initial data data loading in staging table 1 

-- main cycle 
WHILE  1 = 1 
	BEGIN

	-- is the DF switched to staging table 1 ?
	IF (SELECT StagingFlowNr FROM dbo.ETLswitchConfig WHERE ID = 1) = 1  
		BEGIN

		-- control signal to data flow: switch to staging table 2
		UPDATE dbo.ETLswitchConfig SET StagingFlowNr =2 WHERE ID = 1;	 

		-- check if the data transfer to staging table 2 has started 
		WHILE NOT EXISTS (SELECT TOP 1 1 FROM dbo.SalesOrderDetailStg2)	 
			BEGIN 
			WAITFOR DELAY '00:00:01'
			-- has the data flow task completed ?
			IF (SELECT ETLstatus FROM dbo.ETLswitchConfig WHERE ID=1) = 'STOP' 
				GOTO lblEnd;
			END

		-- the data flow with staging table 2 is on
		-- the fresh staged data from staging table 1 are inserted into the destination
		INSERT INTO dbo.SalesOrderDetailHist
			(SalesOrderID, SalesOrderDetailID, CarrierTrackingNumber, 
			OrderQty, ProductID, SpecialOfferID, UnitPrice, UnitPriceDiscount, 
			LineTotal, ModifiedDate)
		SELECT SalesOrderID, SalesOrderDetailID, CarrierTrackingNumber, 
			OrderQty, ProductID, SpecialOfferID, UnitPrice, UnitPriceDiscount, 
			LineTotal, ModifiedDate 
		FROM dbo.SalesOrderDetailStg1;

		-- logging of the insertion results
		INSERT INTO dbo.ETLswitchLog (StagingFlowNr, RecordsStaged)
			VALUES (1, (SELECT count(1) from dbo.SalesOrderDetailStg1));

		-- truncation of staging table 1
		TRUNCATE TABLE  dbo.SalesOrderDetailStg1;
	
		END

	ELSE
		-- same process with staging tables swapped out
		IF (SELECT StagingFlowNr FROM dbo.ETLswitchConfig WHERE ID = 1) = 2
			BEGIN

			UPDATE dbo.ETLswitchConfig SET StagingFlowNr =1 WHERE ID = 1;

			WHILE NOT EXISTS (SELECT TOP 1 1 FROM dbo.SalesOrderDetailStg1)
				BEGIN 
				WAITFOR DELAY '00:00:01'
				-- has the data flow task completed ?
				IF (SELECT ETLstatus FROM dbo.ETLswitchConfig WHERE ID=1) = 'STOP'
					GOTO lblEnd;
				END

			INSERT INTO dbo.SalesOrderDetailHist
				(SalesOrderID, SalesOrderDetailID, CarrierTrackingNumber, 
				OrderQty, ProductID, SpecialOfferID, UnitPrice, UnitPriceDiscount, 
				LineTotal, ModifiedDate)
			SELECT SalesOrderID, SalesOrderDetailID, CarrierTrackingNumber, 
				OrderQty, ProductID, SpecialOfferID, UnitPrice, UnitPriceDiscount, 
				LineTotal, ModifiedDate 
			FROM dbo.SalesOrderDetailStg2;

			INSERT INTO dbo.ETLswitchLog (StagingFlowNr, RecordsStaged)
				VALUES (2, (SELECT count(1) from dbo.SalesOrderDetailStg2));

			TRUNCATE TABLE  dbo.SalesOrderDetailStg2;
		
			END


	END 

lblEnd:

-- insertion of the satging data remained after data flow is over

INSERT INTO dbo.SalesOrderDetailHist
	(SalesOrderID, SalesOrderDetailID, CarrierTrackingNumber, 
	OrderQty, ProductID, SpecialOfferID, UnitPrice, UnitPriceDiscount, 
	LineTotal, ModifiedDate)
SELECT SalesOrderID, SalesOrderDetailID, CarrierTrackingNumber, 
	OrderQty, ProductID, SpecialOfferID, UnitPrice, UnitPriceDiscount, 
	LineTotal, ModifiedDate
FROM dbo.SalesOrderDetailStg1
UNION ALL
SELECT SalesOrderID, SalesOrderDetailID, CarrierTrackingNumber, 
	OrderQty, ProductID, SpecialOfferID, UnitPrice, UnitPriceDiscount, 
	LineTotal, ModifiedDate
FROM dbo.SalesOrderDetailStg2;

-- final logging

INSERT INTO dbo.ETLswitchLog (StagingFlowNr, RecordsStaged)
VALUES 
	(
	 (SELECT IIF(StagingFlowNr=1,2,1) FROM  dbo.ETLswitchConfig WHERE ID =1), 
	 (SELECT count(1) from dbo.SalesOrderDetailStg2 )+ 
	 (SELECT count(1) from dbo.SalesOrderDetailStg1)
	 );

UPDATE dbo.ETLswitchConfig  SET RecordsTransferred =
	(SELECT count(1) from dbo.SalesOrderDetailHist) 
WHERE ID =1 ;

END

GO


